@@ -1,4 +1,5 @@
 {-# LANGUAGE BlockArguments #-}
+{-# OPTIONS_GHC -Wno-unused-do-bind #-}
 {-# LANGUAGE BangPatterns #-}
 {-# LANGUAGE DataKinds #-}
 {-# LANGUAGE TypeOperators #-}
@@ -11,16 +12,20 @@
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE DuplicateRecordFields #-}
 {-# LANGUAGE TypeApplications #-}
+{-# LANGUAGE RankNTypes #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 {-# OPTIONS_GHC -Wno-orphans #-}
 {-# OPTIONS_GHC -Wno-name-shadowing #-}
-{-# OPTIONS_GHC -Wno-unused-do-bind #-}
 
 module Test.MockCat.TypeClassSpec (spec) where
 
-import Data.Text (Text)
-import Test.Hspec (Spec)
+import Data.Text (Text, pack)
+import qualified Data.List as List
+import Test.Hspec (Spec, shouldBe, shouldThrow, Selector, describe, it)
+import Control.Exception (ErrorCall, displayException)
 import Test.MockCat
 import Prelude hiding (readFile, writeFile, any)
+
 import Data.Data
 import Data.List (find)
 import GHC.TypeLits (KnownSymbol, symbolVal)
@@ -88,48 +93,48 @@ _ask ::
   , Typeable env
   ) =>
   params ->
-  MockT m env
+  MockT m (MockResult (Verify.ResolvableParamsOf env))
 _ask p = MockT $ do
   mockInstance <- unMockT $ mock (label "ask") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "ask") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _readFile ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (FilePath -> Text)
   , MonadIO m
   ) =>
   params ->
-  MockT m (FilePath -> Text)
+  MockT m (MockResult (Verify.ResolvableParamsOf (FilePath -> Text)))
 _readFile p = MockT $ do
   mockInstance <- unMockT $ mock (label "readFile") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "readFile") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _writeFile ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (FilePath -> Text -> ())
   , MonadIO m
   ) =>
   params ->
-  MockT m (FilePath -> Text -> ())
+  MockT m (MockResult (Verify.ResolvableParamsOf (FilePath -> Text -> ())))
 _writeFile p = MockT $ do
   mockInstance <- unMockT $ mock (label "writeFile") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "writeFile") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _post ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (Text -> ())
   , MonadIO m
   ) =>
   params ->
-  MockT m (Text -> ())
+  MockT m (MockResult (Verify.ResolvableParamsOf (Text -> ())))
 _post p = MockT $ do
   mockFn <- unMockT $ mock (label "post") p
   ensureVerifiable mockFn
   addDefinition (Definition (Proxy :: Proxy "post") mockFn NoVerification)
-  pure mockFn
+  pure (MockResult ())
 
 findParam :: KnownSymbol sym => Proxy sym -> [Definition] -> Maybe a
 findParam pa definitions = do
@@ -297,12 +302,12 @@ _getBy ::
   , Verify.ResolvableParamsOf (String -> m Int) ~ Param String
   ) =>
   params ->
-  MockT m (String -> m Int)
+  MockT m (MockResult (Verify.ResolvableParamsOf (String -> m Int)))
 _getBy p = MockT $ do
   mockInstance <- unMockT $ mock (label "_getBy") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_getBy") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _echo ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
@@ -311,12 +316,12 @@ _echo ::
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
   ) =>
   params ->
-  MockT m (String -> m ())
+  MockT m (MockResult (Verify.ResolvableParamsOf (String -> m ())))
 _echo p = MockT $ do
   mockInstance <- unMockT $ mock (label "_echo") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_echo") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _fn2_1Sub ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
@@ -325,12 +330,12 @@ _fn2_1Sub ::
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
   ) =>
   params ->
-  MockT m (String -> m ())
+  MockT m (MockResult (Verify.ResolvableParamsOf (String -> m ())))
 _fn2_1Sub p = MockT $ do
   mockInstance <- unMockT $ mock (label "_fn2_1Sub") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fn2_1Sub") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _fn2_2Sub ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
@@ -339,12 +344,12 @@ _fn2_2Sub ::
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
   ) =>
   params ->
-  MockT m (String -> m ())
+  MockT m (MockResult (Verify.ResolvableParamsOf (String -> m ())))
 _fn2_2Sub p = MockT $ do
   mockInstance <- unMockT $ mock (label "_fn2_2Sub") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fn2_2Sub") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _fn3_1Sub ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
@@ -353,12 +358,12 @@ _fn3_1Sub ::
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
   ) =>
   params ->
-  MockT m (String -> m ())
+  MockT m (MockResult (Verify.ResolvableParamsOf (String -> m ())))
 _fn3_1Sub p = MockT $ do
   mockInstance <- unMockT $ mock (label "_fn3_1Sub") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fn3_1Sub") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _fn3_2Sub ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
@@ -367,12 +372,12 @@ _fn3_2Sub ::
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
   ) =>
   params ->
-  MockT m (String -> m ())
+  MockT m (MockResult (Verify.ResolvableParamsOf (String -> m ())))
 _fn3_2Sub p = MockT $ do
   mockInstance <- unMockT $ mock (label "_fn3_2Sub") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fn3_2Sub") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _fn3_3Sub ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
@@ -381,12 +386,12 @@ _fn3_3Sub ::
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
   ) =>
   params ->
-  MockT m (String -> m ())
+  MockT m (MockResult (Verify.ResolvableParamsOf (String -> m ())))
 _fn3_3Sub p = MockT $ do
   mockInstance <- unMockT $ mock (label "_fn3_3Sub") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fn3_3Sub") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _getValueBy ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m String)
@@ -395,12 +400,12 @@ _getValueBy ::
   , Verify.ResolvableParamsOf (String -> m String) ~ Param String
   ) =>
   params ->
-  MockT m (String -> m String)
+  MockT m (MockResult (Verify.ResolvableParamsOf (String -> m String)))
 _getValueBy p = MockT $ do
   mockInstance <- unMockT $ mock (label "_getValueBy") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_getValueBy") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _fnState ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (Maybe s -> m s)
@@ -410,11 +415,11 @@ _fnState ::
   , Verify.ResolvableParamsOf (Maybe s -> m s) ~ Param (Maybe s)
   ) =>
   params ->
-  MockT m (Maybe s -> m s)
+  MockT m (MockResult (Verify.ResolvableParamsOf (Maybe s -> m s)))
 _fnState p = MockT $ do
   mockInstance <- unMockT $ mock (label "_fnState") p
   addDefinition (Definition (Proxy :: Proxy "_fnState") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _fnState2 ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
@@ -423,11 +428,11 @@ _fnState2 ::
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
   ) =>
   params ->
-  MockT m (String -> m ())
+  MockT m (MockResult (Verify.ResolvableParamsOf (String -> m ())))
 _fnState2 p = MockT $ do
   mockInstance <- unMockT $ mock (label "_fnState2") p
   addDefinition (Definition (Proxy :: Proxy "_fnState2") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _fnParam3_1 ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (Int -> Bool -> m String)
@@ -436,12 +441,12 @@ _fnParam3_1 ::
   , Verify.ResolvableParamsOf (Int -> Bool -> m String) ~ (Param Int :> Param Bool)
   ) =>
   params ->
-  MockT m (Int -> Bool -> m String)
+  MockT m (MockResult (Verify.ResolvableParamsOf (Int -> Bool -> m String)))
 _fnParam3_1 p = MockT $ do
   mockInstance <- unMockT $ mock (label "_fnParam3_1") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fnParam3_1") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _fnParam3_2 ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (m Int)
@@ -450,12 +455,12 @@ _fnParam3_2 ::
   , Verify.ResolvableParamsOf (m Int) ~ ()
   ) =>
   params ->
-  MockT m (m Int)
+  MockT m (MockResult (Verify.ResolvableParamsOf (m Int)))
 _fnParam3_2 p = MockT $ do
   mockInstance <- unMockT $ mock (label "_fnParam3_2") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fnParam3_2") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _fnParam3_3 ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (m Bool)
@@ -464,12 +469,12 @@ _fnParam3_3 ::
   , Verify.ResolvableParamsOf (m Bool) ~ ()
   ) =>
   params ->
-  MockT m (m Bool)
+  MockT m (MockResult (Verify.ResolvableParamsOf (m Bool)))
 _fnParam3_3 p = MockT $ do
   mockInstance <- unMockT $ mock (label "_fnParam3_3") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fnParam3_3") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _getByExplicit ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m Int)
@@ -478,12 +483,12 @@ _getByExplicit ::
   , Verify.ResolvableParamsOf (String -> m Int) ~ Param String
   ) =>
   params ->
-  MockT m (String -> m Int)
+  MockT m (MockResult (Verify.ResolvableParamsOf (String -> m Int)))
 _getByExplicit p = MockT $ do
   mockInstance <- unMockT $ mock (label "_getByExplicit") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_getByExplicit") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _echoExplicit ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
@@ -492,12 +497,12 @@ _echoExplicit ::
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
   ) =>
   params ->
-  MockT m (String -> m ())
+  MockT m (MockResult (Verify.ResolvableParamsOf (String -> m ())))
 _echoExplicit p = MockT $ do
   mockInstance <- unMockT $ mock (label "_echoExplicit") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_echoExplicit") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _defaultAction ::
   ( MonadIO m
@@ -506,12 +511,12 @@ _defaultAction ::
   , Typeable a
   ) =>
   params ->
-  MockT m a
+  MockT m (MockResult (Verify.ResolvableParamsOf a))
 _defaultAction p = MockT $ do
   mockInstance <- unMockT $ mock (label "_defaultAction") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_defaultAction") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _produce ::
   ( MockDispatch (IsMockSpec p) p (MockT m) (ResultType m)
@@ -521,12 +526,12 @@ _produce ::
   , Typeable (ResultType m)
   ) =>
   p ->
-  MockT m (ResultType m)
+  MockT m (MockResult (Verify.ResolvableParamsOf (ResultType m)))
 _produce p = MockT $ do
   mockInstance <- unMockT $ mock (label "_produce") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_produce") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 instance MonadIO m => Teletype (MockT m) where
   readTTY = MockT do
@@ -550,12 +555,12 @@ _readTTY ::
   , Verify.ResolvableParamsOf (m String) ~ ()
   ) =>
   params ->
-  MockT m (m String)
+  MockT m (MockResult (Verify.ResolvableParamsOf (m String)))
 _readTTY p = MockT $ do
   mockInstance <- unMockT $ mock (label "_readTTY") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_readTTY") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _writeTTY ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
@@ -564,24 +569,24 @@ _writeTTY ::
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
   ) =>
   params ->
-  MockT m (String -> m ())
+  MockT m (MockResult (Verify.ResolvableParamsOf (String -> m ())))
 _writeTTY p = MockT $ do
   mockInstance <- unMockT $ mock (label "_writeTTY") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_writeTTY") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 _processPost ::
   ( MockDispatch (IsMockSpec params) params (MockT m) (Post -> Bool)
   , MonadIO m
   ) =>
   params ->
-  MockT m (Post -> Bool)
+  MockT m (MockResult (Verify.ResolvableParamsOf (Post -> Bool)))
 _processPost p = MockT $ do
   mockInstance <- unMockT $ mock (label "_processPost") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_processPost") mockInstance NoVerification)
-  pure mockInstance
+  pure (MockResult ())
 
 instance MonadIO m => UserDefinedClass (MockT m) where
   processPost p = MockT do
@@ -592,8 +597,54 @@ instance MonadIO m => UserDefinedClass (MockT m) where
       !result = mockFn p
     lift $ pure result
 
+-- Handwritten polymorphic return helpers for testing
+_readFileHandwritten ::
+  ( MockDispatch (IsMockSpec params) params (MockT IO) (FilePath -> Text)
+  ) =>
+  params ->
+  MockT IO (MockResult (Verify.ResolvableParamsOf (FilePath -> Text)))
+_readFileHandwritten p = do
+  fn <- mock (label "readFile") p :: MockT IO (FilePath -> Text)
+  addDefinition (Definition (Proxy :: Proxy "readFile") fn NoVerification)
+  pure (MockResult ())
+
+_writeFileHandwritten ::
+  ( MockDispatch (IsMockSpec params) params (MockT IO) (FilePath -> Text -> ())
+  ) =>
+  params ->
+  MockT IO (MockResult (Verify.ResolvableParamsOf (FilePath -> Text -> ())))
+_writeFileHandwritten p = do
+  fn <- mock (label "writeFile") p :: MockT IO (FilePath -> Text -> ())
+  addDefinition (Definition (Proxy :: Proxy "writeFile") fn NoVerification)
+  pure (MockResult ())
+
+-- Helper for error matching (same logic as TypeClassCommonSpec)
+missingCallHandwritten :: String -> Selector ErrorCall
+missingCallHandwritten name err =
+  let needle1 = "function `" <> name <> "` was not called the expected number of times."
+      needle2 = "function `_" <> name <> "` was not called the expected number of times."
+   in (needle1 `List.isInfixOf` displayException err) || (needle2 `List.isInfixOf` displayException err)
+
 spec :: Spec
 spec = do
+  -- Handwritten polymorphic return tests
+  describe "Handwritten polymorphic return tests" do
+    it "Program reads content and writes it to output path (expects not used)" do
+      result <- runMockT do
+        _readFileHandwritten $ "input.txt" ~> pack "content"
+        _writeFileHandwritten $ "output.txt" ~> pack "content" ~> ()
+        SpecCommon.operationProgram "input.txt" "output.txt"
+
+      result `shouldBe` ()
+
+    it "Error when read stub is defined but target function readFile is never called (expects used)" do
+      (runMockT @IO do
+        _readFileHandwritten ("input.txt" ~> pack "content")
+          `expects` do
+            called once
+        -- readFile is never called
+        pure ()) `shouldThrow` missingCallHandwritten "readFile"
+  
   -- build SpecDeps and call aggregated spec entrypoint
   let deps = SpecCommon.SpecDeps
         { SpecCommon.basicDeps          = SpecCommon.BasicDeps _readFile _writeFile
